#!/bin/bash

PATH=/usr/sbin:/usr/bin:/sbin:/bin

[[ -f ${NEWROOT:-/sysroot}/proc ]] && exit 0

. /lib/dracut-lib.sh

modname="atasx"

mode="tty"
# plymouth ask-for-password is buggy when using rhgb, so disable it
#type plymouth >& /dev/null && plymouth --ping 2>/dev/null && mode="plymouth"
type plymouth >& /dev/null && plymouth --ping 2>/dev/null && \
	( plymouth hide-splash ; plymouth quit )

debug() {
	echo "<28>${modname} debug: $*" > /dev/kmsg
}

printx() {
	case "${mode}" in
		plymouth) plymouth display-message --text "$*";;
		tty) echo "${modname}: $*" >&2;;
	esac
}

readx() {
	case "${mode}" in
		plymouth) plymouth ask-for-password --prompt "$*";;
		tty) echo "${modname}: $*" >&2; read -s -r tmp < /dev/console; echo ${tmp};;
	esac
}

{ flock -s 9
declare -a rescan

phrase=
for d in /sys/block/[shv]d[a-z]; do
	d=${d##*/}
	hdparm -I /dev/${d} | sed -n -r '/^Security:/,${/^\s+locked$/q1}' && continue
	read model < /sys/block/${d}/device/model
	prompt="disk ${d} (${model}) is locked, enter passphrase to unlock"
	prompt2="(enter '.' to use previous passphrase)"
	RET=1
	while [[ $RET != 0 ]]; do
		p=${prompt}
		[[ -n "${phrase}" ]] && p="${p} ${prompt2}"
		tmp=$(readx "${p}")
		if [[ -z "${tmp}" ]]; then
			printx "skipping..."
			continue 2
		fi
		[[ -n "${phrase}" && "${tmp}" == "." ]] || phrase=${tmp}
		hdparm --security-unlock "${phrase}" /dev/${d}
		sleep 1
		hdparm -I /dev/${d} | sed -n -r '/^Security:/,${/^\s+locked$/q1}'
		RET=$?
		case "$RET" in
			1) printx "device is still locked. Wrong passphrase?";;
			0) rescan+=(${d});;
		esac
	done
done
} 9>/.console.lock

if [[ ${#rescan[@]} > 0 ]]; then
	sleep 2
	for d in ${rescan[@]}; do
		printx "rescan ${d}.."
		hdparm -z /dev/${d}
	done
	type lvm >& /dev/null
	if [[ $? == 0 ]]; then
		printx "lvm rescan.."
		sleep 2
		lvm pvscan --cache -a ay --noudevsync
	fi
fi

